# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from . import util


@util.os_parametrize('osxml', filter_media=True)
def test_existing_variant_for_media(osxml):
    variants = osxml.variants
    for media in osxml.medias:
        variant = media.variant
        if variant and variant not in variants:
            raise AssertionError("variants: media in OS %s has variant=%s "
                                 "but declared variants are %s" %
                                 (osxml.shortid, variant, variants))


@util.os_parametrize('osxml', filter_trees=True)
def test_existing_variant_for_trees(osxml):
    variants = osxml.variants
    for tree in osxml.trees:
        variant = tree.variant
        if variant and variant not in variants:
            raise AssertionError("variants: tree in OS %s has variant=%s "
                                 "but declared variants are %s" %
                                 (osxml.shortid, variant, variants))



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import re

from . import util


DEVICE_MAP = {d.internal_id: d for d in util.DataFiles.devices()}
DEVICE_MAP_SRC = {d.internal_id: d for d in util.SourceFiles.devices()}


def _check_duplicate_devices(osxml):
    """
    Ensure an OS doesn't list a device that's defined in the parent
    """
    broken = []
    related = util.DataFiles.getosxml_related(osxml)
    for osxml2 in related:
        if osxml2.devices is not None:
            for device in osxml2.devices:
                if device in osxml.devices:
                    broken.append(device)
    assert broken == []


def _check_uncommented_devices(osxml):
    """
    Ensure every device listed in the XML is followed by a comment with
    the device string name in it. This helps readability/grepability
    """
    badlines = []
    devlines = [l for l in open(osxml.filename).read().splitlines() if
                "<device id" in l]

    for devid in osxml.devices:
        devname = DEVICE_MAP_SRC[devid].name
        for devline in devlines:
            if devid not in devline:
                continue
            if not re.search(r"<!--.*%s.*-->" % devname, devline):
                badlines.append(devline)

    if badlines:
        raise AssertionError("shortid=%s device lines don't contain a "
                "comment with the device name:\n%s" %
                (osxml.shortid, badlines))


@util.os_parametrize('osxml', filter_devices=True)
def test_devices_duplication(osxml):
    _check_duplicate_devices(osxml)


@util.os_sources_parametrize('osxml', filter_devices=True)
def test_devices_comments(osxml):
    _check_uncommented_devices(osxml)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import locale
import os


def pytest_addoption(parser):
    parser.addoption("--network-tests", action="store_true", default=False,
            help=("Run osinfo-db network tests. Same tests as triggered "
                  "by setting env variable OSINFO_DB_NETWORK_TESTS"))


def pytest_configure(config):
    key = "INTERNAL_OSINFO_DB_DATA_DIR"
    if key not in os.environ:
        os.environ[key] = os.path.realpath(os.path.join(
            os.path.dirname(__file__), "..", "data"))

    key = "INTERNAL_OSINFO_DB_DATA_SRC_DIR"
    if key not in os.environ:
        os.environ[key] = os.path.realpath(os.path.join(
            os.path.dirname(__file__), "..", "data"))

    # Needed for test reproducibility on any system not using a UTF-8 locale
    locale.setlocale(locale.LC_ALL, 'C')
    locale.setlocale(locale.LC_CTYPE, 'en_US.UTF-8')

    # Default to --log-level=info if not otherwise specified
    if (hasattr(config.option, "log_level") and
        config.option.log_level is None):
        config.option.log_level = "info"

    # This will trigger some DATA_DIR validation
    from . import util
    dummy = util


def pytest_ignore_collect(path, config):
    """
    Entirely skip loading test_urls.py if the option wasn't specified
    """
    run_network = bool(config.getoption("--network-tests") or
            os.environ.get("OSINFO_DB_NETWORK_TESTS"))
    if os.path.basename(str(path)) == "test_urls.py" and not run_network:
        return True



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import os

from . import util


def _test_validate_ids(xml, entity_type):
    """
    Ensure the ids are the ones supported by OsinfoLoader.

    This check tries to mimic, in a pythonic way, the very same
    check done by OsinfoLoader::osinfo_loader_check_id()
    """
    relpath = os.path.relpath(xml.filename)
    expected_filename = relpath.split(entity_type + "/")[1]

    extension = bool(".d/" in expected_filename)

    suffix = xml.internal_id[len('http://'):]
    vendor = suffix.split('/', 1)[0]
    entity_name = suffix.split('/', 1)[1].replace("/", "-")

    if extension:
        filename = vendor + "/" + entity_name + ".d"
        assert filename == expected_filename.rsplit("/", 1)[0]
    else:
        filename = vendor + "/" + entity_name + ".xml"
        assert filename == expected_filename



@util.os_parametrize('osxml')
def test_validate_os_ids(osxml):
    """
    Ensure the OS ids are the ones supported by OsinfoLoader.
    """
    return _test_validate_ids(osxml, "os")


@util.device_parametrize('devicexml')
def test_validate_device_ids(devicexml):
    """
    Ensure the Device ids are the ones supported by OsinfoLoader.
    """
    return _test_validate_ids(devicexml, "device")


@util.datamap_parametrize('datamapxml')
def test_validate_datamap_ids(datamapxml):
    """
    Ensure the Datamap ids are the ones supported by OsinfoLoader.
    """
    return _test_validate_ids(datamapxml, "datamap")


@util.installscript_parametrize('installscriptxml')
def test_validate_installscript_ids(installscriptxml):
    """
    Ensure the InstallScript ids are the ones supported by OsinfoLoader.
    """
    return _test_validate_ids(installscriptxml, "install-script")


@util.platform_parametrize('platformxml')
def test_validate_platform_ids(platformxml):
    """
    Ensure the Platform ids are the ones supported by OsinfoLoader.
    """
    return _test_validate_ids(platformxml, "platform")



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import re

import lxml.etree as ET


def _cache_property(fn):
    """
    Decorator to use self._cache to cache property lookup results
    """
    def _wrapper(*args):
        self = args[0]
        key = str(fn)
        cache = self._cache  # pylint: disable=protected-access
        if key not in cache:
            cache[key] = fn(*args)
        return cache[key]
    return property(_wrapper)


class _XMLBase():
    """
    Simple base class for our XML parsers
    """
    def __init__(self, root):
        self._root = root
        self._cache = {}

    def _get_text(self, element_name, default=None):
        node = self._root.find(element_name)
        if node is not None:
            return node.text
        return default

    def _get_int(self, element_name, default=None):
        text = self._get_text(element_name)
        if text is not None:
            return int(text)
        return default


class Os(_XMLBase):
    def __init__(self, filename):
        self.filename = filename
        root = ET.parse(self.filename).getroot().find('os')
        super().__init__(root)

    def __repr__(self):
        return "<%s shortid=%s>" % (self.__class__.__name__, self.shortid)

    @_cache_property
    def internal_id(self):
        return self._root.get('id')

    @_cache_property
    def derives_from(self):
        derives_from = self._root.find('derives-from')
        if derives_from is not None:
            return derives_from.get('id')
        return None

    @_cache_property
    def clones(self):
        clones = self._root.find('clones')
        if clones is not None:
            return clones.get('id')
        return None

    @_cache_property
    def devices(self):
        devices = []
        devicelist = self._root.find('devices')
        if devicelist is not None:
            for device in devicelist.findall('device'):
                devices.append(device.get('id'))
        return devices

    @_cache_property
    def images(self):
        images = []
        for image in self._root.findall('image'):
            images.append(Image(image))
        return images

    @_cache_property
    def medias(self):
        medias = []
        for media in self._root.findall('media'):
            medias.append(Media(media))
        return medias

    @_cache_property
    def trees(self):
        trees = []
        for tree in self._root.findall('tree'):
            trees.append(Tree(tree))
        return trees

    @_cache_property
    def shortid(self):
        return self._get_text('short-id')

    @_cache_property
    def shortids(self):
        shortids = []
        for shortid in self._root.findall('short-id'):
            shortids.append(shortid.text)
        return shortids

    @_cache_property
    def distro(self):
        return self._get_text('distro')

    @_cache_property
    def resources_list(self):
        return self._root.findall('resources')

    def _get_resources(self, node, resource_type):
        valid_resources = ['minimum',
                           'recommended',
                           'maximum',
                           'network-install']
        if resource_type not in valid_resources:
            return None
        # pylint: disable=unsupported-membership-test
        if node not in self.resources_list:
            return None
        resource = node.find(resource_type)
        if resource is not None:
            return Resources(resource)
        return None

    def get_minimum_resources(self, node):
        return self._get_resources(node, 'minimum')

    def get_recommended_resources(self, node):
        return self._get_resources(node, 'recommended')

    def get_maximum_resources(self, node):
        return self._get_resources(node, 'maximum')

    def get_network_install_resources(self, node):
        return self._get_resources(node, 'network-install')

    @_cache_property
    def release_date(self):
        return self._get_text('release-date')

    @_cache_property
    def eol_date(self):
        return self._get_text('eol-date')

    @_cache_property
    def variants(self):
        return [v.attrib['id'] for v in self._root.findall('variant')]

    @_cache_property
    def firmwares(self):
        firmwares = []
        firmwarelist = self._root.findall('firmware')
        for firmware in firmwarelist:
            firmwares.append(Firmware(firmware))
        return firmwares


class Resources(_XMLBase):
    @_cache_property
    def cpu(self):
        return self._get_int('cpu')

    @_cache_property
    def n_cpus(self):
        return self._get_int('n-cpus')

    @_cache_property
    def ram(self):
        return self._get_int('ram')

    @_cache_property
    def storage(self):
        return self._get_int('storage')


class Firmware(_XMLBase):
    @_cache_property
    def arch(self):
        return self._root.attrib["arch"]

    @_cache_property
    def firmware_type(self):
        return self._root.attrib["type"]


class Image(_XMLBase):
    @_cache_property
    def url(self):
        return self._get_text('url')


class Media(_XMLBase):
    @_cache_property
    def url(self):
        return self._get_text('url')

    @_cache_property
    def iso(self):
        iso = self._root.find('iso')
        if iso is not None:
            return ISO(iso)
        return None

    @_cache_property
    def variant(self):
        variant = self._root.find('variant')
        if variant is not None:
            return variant.attrib['id']
        return None


class Tree(_XMLBase):
    @_cache_property
    def url(self):
        return self._get_text('url')

    @_cache_property
    def variant(self):
        variant = self._root.find('variant')
        if variant is not None:
            return variant.attrib['id']
        return None


class ISO(_XMLBase):
    @_cache_property
    def volumeid(self):
        return re.compile(self._get_text('volume-id', default=''))

    @_cache_property
    def publisherid(self):
        return re.compile(self._get_text('publisher-id', default=''))

    @_cache_property
    def applicationid(self):
        return re.compile(self._get_text('application-id', default=''))

    @_cache_property
    def systemid(self):
        return re.compile(self._get_text('system-id', default=''))

    @_cache_property
    def volumesize(self):
        return self._get_int('volume-size', default=0)


class Device(_XMLBase):
    def __init__(self, filename):
        self.filename = filename
        root = ET.parse(self.filename).getroot().find('device')
        super().__init__(root)

    @_cache_property
    def internal_id(self):
        return self._root.attrib["id"]

    @_cache_property
    def name(self):
        return self._get_text('name')


class Datamap(_XMLBase):
    def __init__(self, filename):
        self.filename = filename
        root = ET.parse(self.filename).getroot().find('datamap')
        super().__init__(root)

    def __repr__(self):
        return "<%s filename=%s>" % (self.__class__.__name__, self.filename)

    @_cache_property
    def internal_id(self):
        return self._root.get('id')


class InstallScript(_XMLBase):
    def __init__(self, filename):
        self.filename = filename
        root = ET.parse(self.filename).getroot().find('install-script')
        super().__init__(root)

    def __repr__(self):
        return "<%s filename=%s>" % (self.__class__.__name__, self.filename)

    @_cache_property
    def internal_id(self):
        return self._root.get('id')


class Platform(_XMLBase):
    def __init__(self, filename):
        self.filename = filename
        root = ET.parse(self.filename).getroot().find('platform')
        super().__init__(root)

    def __repr__(self):
        return "<%s filename=%s>" % (self.__class__.__name__, self.filename)

    @_cache_property
    def internal_id(self):
        return self._root.get('id')



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import http
import logging

import pytest
import requests

from . import util


def _check_url(url):
    logging.info("url: %s", url)
    response = requests.head(url, allow_redirects=True)
    logging.info("response: %s; code: %d",
                 http.client.responses[response.status_code],
                 response.status_code)
    return response.ok


def _collect_os_urls():
    """
    Iterate the OS list and return a list of pairs (shortid, [url list)
    """
    ret = []

    for osxml in util.DataFiles.oses():
        urls = []
        urls.extend([i.url for i in osxml.images if i.url])
        urls.extend([m.url for m in osxml.medias if m.url])
        urls.extend([t.url for t in osxml.trees if t.url])
        if urls:
            ret.append((osxml.shortid, urls))

    return ret


@pytest.mark.parametrize('testdata', _collect_os_urls(),
        ids=lambda testdata: testdata[0])
def test_urls(testdata):
    urls = testdata[1]
    broken = []
    for url in urls:
        if not _check_url(url):
            broken.append(url)
    assert broken == []



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import glob
import logging
import os
import pytest

from . import util
from . import isodata


def _get_isodatapaths():
    """
    Collect iso media data and return a list of tuples:
        (osname, isodatapaths)
    """
    isodata_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'isodata')

    ret = []
    allpaths = glob.glob(os.path.join(isodata_path, "*", "*"))
    for osdir in sorted(allpaths, key=util.human_sort):
        osname = os.path.basename(osdir)
        isodatapaths = glob.glob(os.path.join(osdir, "*.txt"))
        if len(isodatapaths):
            ret.append((osname, isodatapaths))
    return ret


@pytest.mark.parametrize("testdata", _get_isodatapaths(), ids=lambda d: d[0])
def test_iso_detection(testdata):
    osname, isodatapaths = testdata
    for isodatapath in isodatapaths:
        detected = []
        isodatamedia = isodata.get_isodatamedia(isodatapath)
        for osxml2 in util.DataFiles.oses():
            for media in osxml2.medias:
                if isodatamedia.match(media.iso):
                    if osname != osxml2.shortid:
                        logging.warning(
                            'ISO \'%s\' was matched by OS \'%s\' while it '
                            'should only be matched by OS \'%s\'',
                            isodatamedia.filename, osxml2.shortid, osname)
                    else:
                        logging.info('ISO \'%s\' matched by OS \'%s\'',
                                     isodatamedia.filename, osxml2.shortid)
                    # For several distros we do not have the volume-size
                    # set as part of our DB, thus multiple detections may
                    # occur. Although this case is not the optimal, as long
                    # as we detect the very same distro it's okay-ish.
                    if osxml2.shortid not in detected:
                        detected.append(osxml2.shortid)

        if detected == [osname]:
            continue

        raise AssertionError("isodata: %s\nMatched=%s but expected=%s" %
                (isodatapath, detected, [osname]))
